// tmwinter.pov
// tom melly - tom@tomandlu.co.uk

#version 3.5;

#include "colors.inc"
#include "functions.inc"
#include "transforms.inc"
#include "strings.inc"

#include "tmtreetex.inc"
#include "tmtree.inc"

#declare doRad    = false;

#declare doTree   = true;
#declare doLeaves = true;
#declare doWall   = true;

#declare doGrass  = true;
#declare dofile   = true;

global_settings {
  assumed_gamma 1.0
  ambient_light 0
  max_trace_level 50
  #if(doRad)
    radiosity {
      pretrace_start    0.08
      pretrace_end      0.02
      count             50
      nearest_count     10 
      error_bound       1
      gray_threshold    0.5 
      brightness        1.0  
      always_sample     off
      normal            on              
    }
  #end
}

sphere{
  0,2000
  inverse
  pigment{
    wrinkles turbulence 0.25 scale 300
    pigment_map{  
      [0.50 function{abs((y+2)/20)} pigment_map{[0.0 rgb<160,190,210>/255][0.1 rgb<98,144,204>/255]}] //rgb<160,160,204>/255
      [0.90 White]
    }
  }
}

//light_source{0 color 1 translate <600,1500,-250>}
light_source {
  0*x                 // light's position (translated below)
  color rgb 1.0       // light's color
  area_light
  <100, 0, 0> <0, 0,100> // lights spread out across this distance (x * z)
  20, 20                // total number of lights in grid (4x*4z = 16 lights)
  adaptive 0          // 0,1,2,3...
  jitter              // adds random softening of light
  circular            // make the shape of the light circular
  orient              // orient light
  translate <600, 1500, -700>   // <x y z> position of light
}

#if(!doRad)
  light_source{0 color 1/5 translate <0,12,-25> shadowless}
#end

camera
{
	location <0,16,-65>
	look_at y*16
	angle 30
}

#declare fn_Land = function(x,y,z){y - f_noise3d(x/1,0,z/1)/5 + f_noise3d(x/15,0,z/15)*5 - 5}
#declare Land =
isosurface {
  function { fn_Land(x,y,z) }
  contained_by { box{<-35,-5,-65>,<35,5,15>}}
  accuracy 0.001
  max_gradient 4
}

#if(doWall)
  #declare WallHi = 7;
  #declare WallStart = -30; //-20
  #declare WallEnd = 30; //-20

  #declare Ground = object{Land translate z*-11}
  MakeWall(WallHi, WallStart, WallEnd, 9732)
  #declare Wall = object{Wall translate z*11 translate y*-0.5};
  object{Wall}

  #declare Ground = object{Land translate z*-12}
  MakeWall(WallHi, WallStart, WallEnd, 11874)
  #declare Wall = object{Wall translate z*12 translate y*-0.5};
  object{Wall}
#end

#if(doGrass)

  #declare p_Size = 10*0.2;
  #declare GRand1 = seed(427);
  #include "tmtreegrass.inc"

  #declare GZ = -55;
  #declare Xtra = 0;
  #declare GNorm = <0,0,0>;
  #while(GZ <= 15)
    #declare GX = -35 - (rand(GRand1)*4 + 1);
    #while(GX <= 35)
      #declare G_Pos = trace(Land, <GX,100,GZ>, <0,-100,0>, GNorm); 
      #if(vlength(GNorm)!=0)
        #if(G_Pos.z > -20)
          object{GrassPatchA rotate y*(int(rand(GRand1)*4)*90) transform{Reorient_Trans(y,GNorm)} translate G_Pos}
        #else
          object{GrassPatchB rotate y*(int(rand(GRand1)*4)*90) transform{Reorient_Trans(y,GNorm)} translate G_Pos}
        #end
      #end
      #declare GX = GX + p_Size;
    #end
    #declare GZ = GZ + p_Size;
  #end

#else
  #declare txtGrass1 =
  texture{
    pigment{rgb<75,125,10>/255}
    finish{specular 0.25 roughness 0.01 diffuse 0.8}
  }
  box{<-30,-1,-55>,<30,0,15> texture{txtGrass1}}
#end

//#macro Make_Tree(TArraySize, TSeed, TBWidth, THt, TCutoff, TBranch, TAng, TLevs)
#if(doTree)
  Make_Tree(50000, 155246, 5, 40, 8, 0.5, 30, 7) 
  #declare Tree = object{Tree scale 0.75 rotate z*-15 rotate x*15}
  object{Tree translate<0,0,4>}
#end

